// scenerytools.hpp -- the toolbar for scenery operations
//
// Written by Frederic Bouvier, started March 2003.
//
// Copyright (C) 2003  Frederic Bouvier - fredb@users.sourceforge.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: scenerytools.hpp,v 1.8 2005/05/09 07:02:13 fredb Exp $

#ifndef _scenerytools_hpp_
#define _scenerytools_hpp_

#include <FL/Fl_Group.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Value_Slider.H>
#include <FL/Fl_Pixmap.H>

class FGSD_MainWindow;
class FGSD_FloatInput;

class FGSD_SceneryToolbar : public Fl_Group {
   public:
      FGSD_SceneryToolbar( int x, int y, int w, int h, FGSD_MainWindow *mw );

      void show( bool yes = true );
      bool drawTiles() const { return _showScenery->value(); }
      void drawTiles( bool d );
      bool drawStatic() const { return _showScenery->value() && _showStatic->value(); }
      void drawStatic( bool d ) { _showStatic->value( d ); }
      bool drawTileFrames() const { return _showScenery->value() && _showFrame->value(); }
      void drawTileFrames( bool d ) { _showFrame->value( d ); }
      bool drawShadows() const;
      bool vertexEdit() const { return _showScenery->value() && _editVertex->value(); }
      void vertexEdit( bool v ) const { _editVertex->value( v ); }
      double alphaValue() const { return _alphaTile->value(); }
      const char *heightValue() const;
      void heightValue( const char *v );
      void noFragmentPresent();
      void fragmentPresent();

      static void drawType_cb( Fl_Widget *w, void *v );

   private:
      Fl_Button *_showScenery;
      Fl_Button *_showFrame;
      Fl_Button *_showStatic;
      Fl_Choice *_viewType;
      Fl_Button *_editVertex;
      Fl_Value_Slider *_alphaTile;
      FGSD_FloatInput *_hentry;
      Fl_Pixmap _shadowImage;
};

#endif
